
// Certain Death Via Space Things
// Phoenix Kokido 2001

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"

#include "KG_Util.h"

struct object;

float sinv[360];
float cosv[360];

/* pre calculated particle colors */
/* White to yellow, to orange, to red, to black */
Uint32 pcolor[500];


typedef struct {
	Uint16 type;
	Uint32 color;
	Uint32 timer;
	float vx, vy;
	float x, y;
	Uint32 stime;
	int t;
} particle;

typedef struct {
	Uint16 type;
	int alive;
	float vx, vy, x, y;
} t_shot;

typedef struct object{
	Uint16 type;
	SDL_Rect pos;
	int alive;
	int state;
	t_shot shots[3];
	Sint32 timer[5];
	Sint32 vx, vy;

	Uint8 *mask;
} object;

SDL_Rect FontData[128];
SDL_Surface *Font;

particle p[500];

Mix_Chunk *explode;
Mix_Chunk *shot_snd;


SDL_Surface *screen;
SDL_Surface *meter;
SDL_Surface *meter2;

SDL_Rect    *stars;
SDL_Surface *tile;
SDL_Surface *ship;
Uint8 *cd_ship;

SDL_Surface *alien;
Uint8 *cd_alien;

SDL_Surface *alien2[2];
Uint8 *cd_alien2[2];

SDL_Surface *alien3[2];
Uint8 *cd_alien3[2];

SDL_Surface *alien4[2];
Uint8 *cd_alien4[2];

SDL_Surface *shot;
Uint8 *cd_shot;

SDL_Surface *powerup[5];
Uint8 *cd_powerup[5];

SDL_Surface *shield;
SDL_Surface *battleship;
object      *game_obs;
int         *tilemap;
Uint8       *keys; 
object      *player;

t_shot *shots;
object *mon;
object *aliens[100];
int psc;
int ac;
int atc[10];
int pc;

Sint32 tiley = 200 * 32;
SDL_Event event;
int done=0;
SDL_Rect dst;
int i, j;
int fire=0;
Uint32 spawntimer = 0;
int score = 0;
int hiscore = 0;

#define FRAMES_PER_SEC 45
#define MAX_CHANNELS   16

void Wait_Frame(void)
{
  static Uint32 next_tick = 0;
  Uint32 this_tick;

  /* Wait for the next frame */
  this_tick = SDL_GetTicks(); 

  if ( this_tick < next_tick ) {
    SDL_Delay(next_tick-this_tick);
  }

  next_tick = this_tick + (1000/FRAMES_PER_SEC);
}

void LoadFont(void) {
	FILE *fp;
		
	int i, x, y, w, h;
	int result;

	Font = Kokido_SafeLoad(DATA_PREFIX "data/droid.png");
	if(!Font) {
		fprintf(stderr, "Error loading font image\n");
	}
	
	fp = fopen(DATA_PREFIX "data/droid.def", "r");

	if(fp!=NULL) {
		result=fscanf(fp, "%i %i %i %i %i\n", &i, &x, &y, &w, &h);
		while(result!=EOF) {
			if(i<128) {
				FontData[i].x=x;
				FontData[i].y=y;
				FontData[i].w=w;
				FontData[i].h=h;
			}
			result=fscanf(fp, "%i %i %i %i %i\n", &i, &x, &y, &w, &h);
		}
		fclose(fp);
	}
	else {
		fprintf(stderr, "Error loading font data\n");
	}
}

Uint8 * CreateCDMask(SDL_Surface *surf) {
	Uint8 *mask;
	int x, y;
	Uint32 pixel;
	int offset=0;
    
	mask = malloc(surf->h * surf->w * sizeof(Uint8)); 

	for(y=0; y<surf->h; y++) {
		for(x=0; x<surf->w; x++) {
			pixel = read_pixel(x,y,surf);
			if(((pixel & surf->format->Amask) >> surf->format->Ashift) > 0x88) {
				mask[offset] = 0xFF;
			}
			else {
				mask[offset] = 0;
			}
			offset++;
		}
	}

	return(mask);
}

void LoadData(void) {
	ship         = Kokido_SafeLoad(DATA_PREFIX "data/ship.png");
	cd_ship = CreateCDMask(ship);

	alien        = Kokido_SafeLoad(DATA_PREFIX "data/alien1.png");
	cd_alien = CreateCDMask(alien);

	alien2[0]    = Kokido_SafeLoad(DATA_PREFIX "data/alien2.png");
	cd_alien2[0] = CreateCDMask(alien2[0]);

	alien2[1]    = Kokido_SafeLoad(DATA_PREFIX "data/alien3.png");
	cd_alien2[1] = CreateCDMask(alien2[1]);

	alien3[1]    = Kokido_SafeLoad(DATA_PREFIX "data/alien4.png");
	cd_alien3[1] = CreateCDMask(alien3[1]);

	alien3[0]    = Kokido_SafeLoad(DATA_PREFIX "data/alien5.png");
	cd_alien3[0] = CreateCDMask(alien3[0]);

	alien4[0]    = Kokido_SafeLoad(DATA_PREFIX "data/spore2.png");
	cd_alien4[0] = CreateCDMask(alien4[0]);

	alien4[1]    = Kokido_SafeLoad(DATA_PREFIX "data/spore2b.png");
	cd_alien4[1] = CreateCDMask(alien4[1]);

	shot         = Kokido_SafeLoad(DATA_PREFIX "data/shot.png");
	cd_shot  = CreateCDMask(shot);

	tile         = Kokido_SafeLoad(DATA_PREFIX "data/tile.png");
	powerup[0]   = Kokido_SafeLoad(DATA_PREFIX "data/powerup.png");
	cd_powerup[0] = CreateCDMask(powerup[0]);

	powerup[1]   = Kokido_SafeLoad(DATA_PREFIX "data/powerup2.png");
	cd_powerup[1] = CreateCDMask(powerup[1]);

	powerup[2]   = Kokido_SafeLoad(DATA_PREFIX "data/powerup3.png");
	cd_powerup[2] = CreateCDMask(powerup[2]);

	powerup[3]   = Kokido_SafeLoad(DATA_PREFIX "data/powerup4.png");
	cd_powerup[3] = CreateCDMask(powerup[3]);

	powerup[4]   = Kokido_SafeLoad(DATA_PREFIX "data/powerup5.png");
	cd_powerup[4] = CreateCDMask(powerup[4]);

	shield       = Kokido_SafeLoad(DATA_PREFIX "data/shield.png");
	battleship   = Kokido_SafeLoad(DATA_PREFIX "data/battleship.png");

	meter        = Kokido_SafeLoad(DATA_PREFIX "data/meter.png");
	meter2       = Kokido_SafeLoad(DATA_PREFIX "data/meter2.png");


	tilemap   = malloc(sizeof(int) * (20 * 200));
	game_obs  = malloc(sizeof(object) * 100);
	stars     = malloc(sizeof(SDL_Rect) * 300);
	shots     = malloc(sizeof(t_shot) * 100);
	explode   = Mix_LoadWAV(DATA_PREFIX "data/bombex.wav");
	shot_snd  = Mix_LoadWAV(DATA_PREFIX "data/zap2.wav");

	LoadFont();

	//Build sin and cos tables
	for(i=0; i<360; i++) {
		sinv[i] = sin((float)i * (3.14159 / 180.0));
		cosv[i] = -cos((float)i * (3.14159 / 180.0));
	}

	for(i=0; i<500; i++) {
		p[i].type = 0;
	}

	/* White to Yellow */
	for(i=0; i<125; i++) {
		pcolor[i] = SDL_MapRGB(screen->format, 0xff, 0xff, (125-i) * 0xff / 125);
	}

	/* Yellow to Orange */
	for(i=0; i<125; i++) {
		pcolor[125 + i] = SDL_MapRGB(screen->format, 0xff, 0x88 + ((125-i) * 0x88 / 125), 0);
	}

	/* Orange to Red*/
	for(i=0; i<125; i++) {
		pcolor[250 + i] = SDL_MapRGB(screen->format, 0xff, ((125-i) * 0x88 / 125), 0);
	}

	/* Red to Black*/
	for(i=0; i<125; i++) {
		pcolor[375 + i] = SDL_MapRGB(screen->format, (125-i) * 0xff / 125, 0, 0);
	}
}

void FreeData(void) {
	SDL_FreeSurface(ship);
	SDL_FreeSurface(alien);
	SDL_FreeSurface(alien2[0]);
	SDL_FreeSurface(alien2[1]);
	SDL_FreeSurface(alien3[0]);
	SDL_FreeSurface(alien3[1]);
	SDL_FreeSurface(shot);
	SDL_FreeSurface(powerup[0]);
	SDL_FreeSurface(powerup[1]);
	SDL_FreeSurface(powerup[2]);
	SDL_FreeSurface(powerup[3]);
	SDL_FreeSurface(powerup[4]);
	SDL_FreeSurface(Font);
	SDL_FreeSurface(battleship);

	free(cd_alien);
	free(cd_shot);

	free(tilemap);
	free(game_obs);
	free(stars);
	free(shots);
}

// Full object-to-object pixel-level collision detector:
int CheckCollision(Uint8 *m1, SDL_Rect *r1,
					Uint8 *m2, SDL_Rect *r2) {
    int left1, left2, over_left;
    int right1, right2, over_right;
    int top1, top2, over_top;
    int bottom1, bottom2, over_bottom;

    int over_width, over_height;
    int i, j;
    Uint8 *pixel1, *pixel2;

    left1 = r1->x;
    left2 = r2->x;
    right1 = r1->x + r1->w;
    right2 = r2->x + r2->w;
    top1 = r1->y;
    top2 = r2->y;
    bottom1 = r1->y + r1->h;
    bottom2 = r2->y + r2->h;

    if (bottom1 < top2) return(0);
    if (top1 > bottom2) return(0);  
    if (right1 < left2) return(0);
    if (left1 > right2) return(0);
    
    if (bottom1 > bottom2) over_bottom = bottom2;
		else over_bottom = bottom1;

    if (top1 < top2) over_top = top2;
		else over_top = top1;
    
	if (right1 > right2) over_right = right2;
		else over_right = right1;
    
	if (left1 < left2) over_left = left2;
		else over_left = left1;

	over_width = over_right - over_left;
	over_height = over_bottom - over_top;

    i = ((over_top - r1->y) * r1->w) + (over_left - r1->x);
    pixel1 = m1 + i;
    j = ((over_top - r2->y) * r2->w) + (over_left - r2->x);
    pixel2 = m2 + j;  

    for (i=0; i < over_height; i++) {
        for (j=0; j < over_width; j++) {
            if (*pixel1 & *pixel2) return(1);
            pixel1++;
            pixel2++;
        }
        pixel1 += (r1->w - over_width);
        pixel2 += (r2->w - over_width);    
	}

	return(0);
};

void blitstr(SDL_Surface *surf, Sint32 x, Sint32 y, char *text) {
	unsigned int i, rval;
	SDL_Rect src, dst;

	dst.x = x;
	dst.y = y;

	for(i=0; i<strlen(text); i++) {
		src = FontData[text[i]];

		dst.w=src.w;
		dst.h=src.h;
			
		rval = SDL_BlitSurface(Font, &src, screen, &dst);

		dst.x+=src.w;
	}

}

void draw_background(void) {
	int i, y, x;

	for(i=0; i<100; i++) {
		stars[i].y += 1;
		while(stars[i].y >= screen->h) stars[i].y -= screen->h;
		SDL_FillRect(screen, &stars[i], SDL_MapRGB(screen->format, 64, 64, 64));
	}
	for(i=100; i<200; i++) {
		stars[i].y += 2;
		while(stars[i].y >=  screen->h) stars[i].y -= screen->h;
		SDL_FillRect(screen, &stars[i], SDL_MapRGB(screen->format, 128, 128, 128));
	}
	for(i=200; i<300; i++) {
		stars[i].y += 3;
		while(stars[i].y >= screen->h) stars[i].y -= screen->h;
		SDL_FillRect(screen, &stars[i], SDL_MapRGB(screen->format, 255, 255, 255));
	}
}

void Explode(int type, SDL_Rect pos) {
	int a,b, c;
	float v;

	if(type==1) {
		for(a=0; a<25; a++) {
			for(b = 0; b < 500; b++) {
				if(p[b].type == 0) {
					p[b].type = 1;
					p[b].x = pos.x;
					p[b].y = pos.y;
					c = rand()%360;
					v = (float)((rand()%100) / 20);
					p[b].vx = sinv[c] * v;
					p[b].vy = cosv[c] * v;
					p[b].color = SDL_MapRGB(screen->format, rand()%255, rand()%255, 0);
					p[b].timer = SDL_GetTicks() + rand()%500;
					p[b].t = rand() % 125;
					break;
				}
			}
		}
	}
	else {
			for(b = 0; b < 500; b++) {
				if(p[b].type == 0) {
					p[b].type = 1;
					p[b].x = pos.x;
					p[b].y = pos.y;
					c = rand()%360;
					v = (float)(rand()%80) / 20.0f;
					p[b].vx = sinv[c] * v;
					p[b].vy = cosv[c] * v;

					//This one is kinda cool :)
					//p[b].vx = sinv[b % 360] * 4.0f;
					//p[b].vy = cosv[b % 360] * 4.0f;
					
					p[b].color = SDL_MapRGB(screen->format, 0xff, 0xff, 0x88);
					p[b].timer = SDL_GetTicks() + rand()%500;
					p[b].t = rand() % 125;
				}
			}
	}

	Mix_PlayChannel(0,explode, 0);

}

void MoveObjects(void) {
	float diff;
	int k;

	psc = 0;
	for(i=0; i<100; i++) {
		switch(game_obs[i].type) {
		case(0): break;
		case(1): break;
		case(2): 
				switch(game_obs[i].state) {
				case(0):
					game_obs[i].pos.y += 4;
					if(game_obs[i].pos.y > game_obs[i].pos.h * 2) { 
						game_obs[i].state = 1;
						game_obs[i].timer[0] = SDL_GetTicks() + 5000 + rand()%1000; 
						game_obs[i].timer[1] = SDL_GetTicks() + rand() % 1000;
						game_obs[i].vx = rand() % 4 - 2;
						game_obs[i].vy = rand() % 4 - 2;
					}
					break;
                case(1):
					if(SDL_GetTicks() > game_obs[i].timer[0]) {
						game_obs[i].state = 2;
					}
					if(SDL_GetTicks() > game_obs[i].timer[1]) {
						game_obs[i].vx = rand() % 4 - 2;
						game_obs[i].vy = rand() % 4 - 2;
						game_obs[i].timer[1] = SDL_GetTicks() + rand() % 1000;

						if(!(rand()%4)) {
							for(j=0; j<100; j++) {
							if(shots[j].type == 0) {
								shots[j].type = 2;
								shots[j].x = game_obs[i].pos.x;
								shots[j].y = game_obs[i].pos.y;
								
								diff = (float)(game_obs[i].pos.x - player->pos.x);
								if(diff < 0) diff *= -1;
								if((game_obs[i].pos.y - player->pos.y) < 0) 
									diff -= (float)(game_obs[i].pos.y - player->pos.y);
								else
									diff += (float)(game_obs[i].pos.y - player->pos.y);
								diff *= -1;

								shots[j].vx = ((float)(game_obs[i].pos.x - player->pos.x) / diff) * 4;
								shots[j].vy = ((float)(game_obs[i].pos.y - player->pos.y) / diff) * 4;
								break;
							}
							}
						}
					}
					game_obs[i].pos.x += game_obs[i].vx;
					game_obs[i].pos.y += game_obs[i].vy;
					if(game_obs[i].pos.x < 0) {
						game_obs[i].pos.x = 0;
						game_obs[i].vx *= -1;
					}
					if(game_obs[i].pos.y < 0) {
						game_obs[i].pos.y = 0;
						game_obs[i].vy *= -1;
					}
					if(game_obs[i].pos.x > screen->w - game_obs[i].pos.w) {
						game_obs[i].pos.x = screen->w - game_obs[i].pos.w;
						game_obs[i].vx *= -1;
					}
					break;
				case(2): 
					if(game_obs[i].pos.x < player->pos.x) game_obs[i].pos.x++;
					if(game_obs[i].pos.x > player->pos.x) game_obs[i].pos.x--;
					game_obs[i].pos.y += 4;
					break;
				default:
					game_obs[i].pos.y += 3;
					break;
				}

			break;
		case(3): 
			game_obs[i].pos.x--;
			if(game_obs[i].pos.x < -game_obs[i].pos.w) game_obs[i].type = 0;
			
			if(SDL_GetTicks() > game_obs[i].timer[1]) {
				if(game_obs[i].state == 0) game_obs[i].state = 1;
				else game_obs[i].state = 0;
			
				game_obs[i].timer[1] = SDL_GetTicks() + 250 + rand()%250;
			}
			
			if(game_obs[i].state == 0) {
				game_obs[i].pos.y--;
			}
			else {
				game_obs[i].pos.y++;
			}
			if(game_obs[i].pos.y < 0) game_obs[i].pos.y = 0;

			if(SDL_GetTicks()> game_obs[i].timer[0]) {
				for(j=0; j<100; j++) {
					if(shots[j].type == 0) {
						shots[j].type = 2;
						shots[j].x = game_obs[i].pos.x + game_obs[i].pos.w / 2;
						shots[j].y = game_obs[i].pos.y + game_obs[i].pos.h;

						shots[j].vx = 0;
						shots[j].vy = 4;
					break;
					}
				}
				game_obs[i].timer[0] = SDL_GetTicks() + 250 + rand()%250;
			}

			break;
		case(5):
			if(game_obs[i].state < 2) {
				if(SDL_GetTicks() > game_obs[i].timer[0]) {
					game_obs[i].state++;
					game_obs[i].timer[0] += 250;
				}
			}
			else {
				if(SDL_GetTicks() > game_obs[i].timer[0]) {
					game_obs[i].timer[0] = SDL_GetTicks() + 2000;
					for(j=0; j<100; j++) {
						if(shots[j].type == 0) {
							shots[j].type = 2;
							shots[j].x = game_obs[i].pos.x;
							shots[j].y = game_obs[i].pos.y;
							
							diff = (float)(game_obs[i].pos.x - player->pos.x);
							if(diff < 0) diff *= -1;
							if((game_obs[i].pos.y - player->pos.y) < 0) 
								diff -= (float)(game_obs[i].pos.y - player->pos.y);
							else
								diff += (float)(game_obs[i].pos.y - player->pos.y);
							diff *= -1;

							shots[j].vx = ((float)(game_obs[i].pos.x - player->pos.x) / diff) * 4;
							shots[j].vy = ((float)(game_obs[i].pos.y - player->pos.y) / diff) * 4;
							break;
						}
					}
				}

			}
			break;
		case(7):
			game_obs[i].pos.y += 3;
			if(game_obs[i].pos.y > screen->h / 2) {
				game_obs[i].type = 0;
				Explode(1, game_obs[i].pos);
				for(k=0; k<10; k++) {
					for(j=0; j<100; j++) {
						if(shots[j].type == 0) {
							shots[j].type = 3;
							shots[j].x = game_obs[i].pos.x;
							shots[j].y = game_obs[i].pos.y;
							
							shots[j].vx = sinv[k * 36] * 4;
							shots[j].vy = cosv[k * 36] * 4;
							break;
						}
					}
				}
			}
			break;
		case(100):
		case(101):
		case(102):
		case(103):
		case(104):
			game_obs[i].pos.y += 3;
			break;

		}
		if(game_obs[i].pos.y > screen->h) game_obs[i].type = 0;
	}

    for(i=0; i<100; i++) {
		if(shots[i].type != 0) {
			if(shots[i].type == 1) psc++;
			shots[i].x += shots[i].vx;
			shots[i].y += shots[i].vy;
			if(shots[i].y < 0) shots[i].type = 0;
			if(shots[i].y > screen->h) shots[i].type = 0;
			if(shots[i].x < 0) shots[i].type = 0;
			if(shots[i].x > screen->w) shots[i].type = 0;
		}
	}

	pc = 0;
	for(i=0; i<500; i++) {
		if(p[i].type != 0) {
			pc++;
			p[i].x += p[i].vx;
			p[i].y += p[i].vy;

			p[i].t+=35;

			if(p[i].t > 499) p[i].t = 499;
			
			if(SDL_GetTicks() > p[i].timer) {
				p[i].type  = 0;
			}
		}
	}
}

void DrawObjects(void) {
	for(i=0; i<100; i++) {
		switch(game_obs[i].type) {
		case(0):break;
		case(1):
			if(player->alive) {
				dst = game_obs[i].pos;
				SDL_BlitSurface(ship, NULL, screen, &dst);

				//dst.x = game_obs[i].pos.x + (ship->w / 2) - (shield->w / 2);
				//dst.y = game_obs[i].pos.y + (ship->h / 2) - (shield->h / 2);
				//dst.w = shield->w;
				//dst.h = shield->h;
				//SDL_BlitSurface(shield, NULL, screen, &dst);

			}

			break;
		case(2):
			dst = game_obs[i].pos;
			SDL_BlitSurface(alien, NULL, screen, &dst);

			break;
		case(3):
			dst = game_obs[i].pos;
			SDL_BlitSurface(alien2[0], NULL, screen, &dst);

			break;
		case(5):
			dst = game_obs[i].pos;
			if(game_obs[i].state == 1) {
				SDL_BlitSurface(alien3[0], NULL, screen, &dst);
			}
			if(game_obs[i].state == 2) {
				SDL_BlitSurface(alien3[1], NULL, screen, &dst);
			}

			break;
		case(7):
			dst = game_obs[i].pos;
			SDL_BlitSurface(alien4[0], NULL, screen, &dst);
			break;
		case(100):
		case(101):
		case(102):
		case(103):
		case(104):
			dst = game_obs[i].pos;
			SDL_BlitSurface(powerup[game_obs[i].type - 100], NULL, screen, &dst);
			break;
		}

		if(shots[i].type != 0) {
			dst.x = shots[i].x;
			dst.y = shots[i].y;
			dst.w = shot->w;
			dst.h = shot->h;
			if(shots[i].type == 3) 
				SDL_BlitSurface(alien4[1], NULL, screen, &dst);
			else
				SDL_BlitSurface(shot, NULL, screen, &dst);
		}
	}

	for(i=0; i<500; i++) {
		if(p[i].type != 0) {
			dst.x = p[i].x;
			dst.y = p[i].y;
			dst.w = 2;
			dst.h = 2;
			SDL_FillRect(screen, &dst, pcolor[p[i].t]);/*p[i].color);*/
		}
	}
}



int CheckCollisions(void) {
	int retval = 0;
	int de = 0;

	ac = 0;
	atc[0] = 0;
	atc[1] = 0;
	atc[2] = 0;
	atc[7] = 0;

	for(i=0; i<100; i++) {
		switch(game_obs[i].type) {
			case(0):break;
			case(1):break;
			case(2):
				atc[0]++;
			case(3):
				if(game_obs[i].type == 3) atc[1]++;
			case(5):
				if(game_obs[i].type == 5) {
					atc[2]++;
	
					if(game_obs[i].state == 0) break;
				}
			case(7):
				if(game_obs[i].type == 7) atc[7]++;


				aliens[ac++] = &game_obs[i];
				if(CheckCollision(game_obs[i].mask, &game_obs[i].pos, cd_ship, &player->pos)) {
					if(player->alive)
						retval = 1;		
				}
				break;
			case(100):
				if(CheckCollision(cd_ship, &player->pos, cd_powerup[0], &game_obs[i].pos)) {
					game_obs[i].type = 0;
					de = 1;
				}
				break;
			case(101):
				if(CheckCollision(cd_ship, &player->pos, cd_powerup[0], &game_obs[i].pos)) {
					game_obs[i].type = 0;
					player->state |= 1;
				}
				break;

			case(102):
				if(CheckCollision(cd_ship, &player->pos, cd_powerup[0], &game_obs[i].pos)) {
					game_obs[i].type = 0;
					player->state |= 2;
				}
				break;
			case(103):
			case(104):
				if(CheckCollision(cd_ship, &player->pos, cd_powerup[0], &game_obs[i].pos)) {
					game_obs[i].type = 0;
					de = 1;
				}
				break;
		}
	}
	if(de) {
		atc[0] = 0;
		atc[1] = 0;
		atc[2] = 0;
		for(i=0; i<ac; i++) {
			aliens[i]->type = 0;
			Explode(1, aliens[i]->pos);
			score++;
		}
		ac = 0;
	}

	for(i=0; i<100; i++) {
		if(shots[i].type != 0) {
			switch(shots[i].type) {
			case(1):
			case(4):
				for(j=0; j<ac; j++) {
					dst.x = shots[i].x;
					dst.y = shots[i].y;
					dst.w = shot->w;
					dst.h = shot->h;

					if(CheckCollision(cd_shot, &dst, aliens[j]->mask, &aliens[j]->pos)) {
						shots[i].type = 0;
						aliens[j]->type = 0;
						Explode(1, aliens[j]->pos);
						score++;

						if(player->state & 2) {
								for(i=0; i<100; i++) {
									if( shots[i].type == 0) {
										shots[i].type = 4;
										shots[i].x = aliens[j]->pos.x + aliens[j]->pos.w / 2  - 2;
										shots[i].y = aliens[j]->pos.y - 2;
										shots[i].vx = sinv[270]*4;
										shots[i].vy = cosv[270]*4;
										shots[i].alive = 1;
								
										break;
									}
								}
								for(i=0; i<100; i++) {
									if( shots[i].type == 0) {
										shots[i].type = 4;
										shots[i].x = aliens[j]->pos.x + aliens[j]->pos.w / 2  - 2;
										shots[i].y = aliens[j]->pos.y - 2;
										shots[i].vx = sinv[315]*4;
										shots[i].vy = cosv[315]*4;
										shots[i].alive = 1;
								
										break;
									}
								}
								for(i=0; i<100; i++) {
									if( shots[i].type == 0) {
										shots[i].type = 4;
										shots[i].x = aliens[j]->pos.x + aliens[j]->pos.w / 2  - 2;
										shots[i].y = aliens[j]->pos.y - 2;
										shots[i].vx = sinv[45]*4;
										shots[i].vy = cosv[45]*4;
										shots[i].alive = 1;
								
										break;
									}
								}
								for(i=0; i<100; i++) {
									if( shots[i].type == 0) {
										shots[i].type = 4;
										shots[i].x = aliens[j]->pos.x + aliens[j]->pos.w / 2  - 2;
										shots[i].y = aliens[j]->pos.y - 2;
										shots[i].vx = sinv[90]*4;
										shots[i].vy = cosv[90]*4;
										shots[i].alive = 1;
								
										break;
									}
								}
						}
					}
				}
				break;
			case(2):
			case(3):
				dst.x = shots[i].x;
				dst.y = shots[i].y;
				dst.w = shot->w;
				dst.h = shot->h;

				if(CheckCollision(cd_shot, &dst, cd_ship, &player->pos)) {
					shots[i].type = 0;
					if(player->alive)
						retval = 1;
				}
				break;
			}
		}
	}

	return(retval);
}

void RunGame(void) {
	char buffer[255];
	int k;
	Uint32 pupspawn;
	int wasteshots = 0;
	Uint32 starttime;

	SDL_Rect src;

	score = 0;
	done = 0;


	for(i=0; i<20; i++) {
		for(j=0; j<200; j++) {
			if(!(rand() % 10) ) {
				tilemap[j * 20 + i] = 1;
			}
			else {
				tilemap[j * 20 + i] = 0;
			}
		}
	}

	for(i=0; i<5; i++) {
		shots[i].alive = 0;
	}

	for(i=0; i<300; i++) {
		stars[i].w = 1;
		stars[i].h = 1;
		stars[i].x = rand()%screen->w;
		stars[i].y = rand()%screen->h;
	}

	for(i=0; i<100; i++) {
		game_obs[i].type = 0;
		shots[i].type = 0;
	}

	dst.x = screen->w / 2 - ship->w / 2;
	dst.y = screen->h - ship->h * 2;
	dst.w = ship->w;
	dst.h = ship->h;

	player = &game_obs[0];
	player->pos = dst;
	player->type = 1;
	player->alive = 1;
	player->state = 0;
	player->timer[0] = 0;

	pupspawn = SDL_GetTicks() + 10000 + (rand()%5000);

	starttime = SDL_GetTicks();

	do {
		while(SDL_PollEvent(&event)) {
			if ( event.type == SDL_QUIT )
				done=1;
		}

		keys = SDL_GetKeyState(NULL); 


		if(keys[SDLK_ESCAPE] == SDL_PRESSED ) {
			done=1;
		}

		if(keys[SDLK_LCTRL] == SDL_PRESSED ) {
			if(player->timer[0] == 300) {
				wasteshots = 1;
				for(i=0; i<100; i++) {
					if(shots[i].type > 1) shots[i].type = 0;
				}
				player->timer[0] = 0;
			}
		}

		if(keys[SDLK_LEFT] == SDL_PRESSED ) {
			player->pos.x -= 2;
			if(player->pos.x < 0) player->pos.x = 0;
		}
		if(keys[SDLK_RIGHT] == SDL_PRESSED ) {
			player->pos.x += 2;
			if(player->pos.x + player->pos.w > screen->w) player->pos.x = screen->w - player->pos.w;
		}
		if(keys[SDLK_SPACE] == SDL_PRESSED ) {
			if(player->alive) {
				if(!fire) {
					if(player->state & 1) {
						if(psc < 15) {
							for(i=0; i<100; i++) {
								if( shots[i].type == 0) {
									shots[i].type = 1;
									shots[i].x = player->pos.x + player->pos.w / 2  - 2;
									shots[i].y = player->pos.y - 2;
									shots[i].vx = sinv[345]*4;
									shots[i].vy = cosv[345]*4;
									shots[i].alive = 1;
						
									break;
								}
							}
							for(; i<100; i++) {
								if( shots[i].type == 0) {
									shots[i].type = 1;
									shots[i].x = player->pos.x + player->pos.w / 2  - 2;
									shots[i].y = player->pos.y - 2;
									shots[i].vx = sinv[0]*4;
									shots[i].vy = cosv[0]*4;
									shots[i].alive = 1;
								
									break;
								}
							}
							for(; i<100; i++) {
								if( shots[i].type == 0) {
									shots[i].type = 1;
									shots[i].x = player->pos.x + player->pos.w / 2  - 2;
									shots[i].y = player->pos.y - 2;
									shots[i].vx = sinv[15]*4;
									shots[i].vy = cosv[15]*4;
									shots[i].alive = 1;
								
									Mix_PlayChannel(1,shot_snd, 0);
								
									break;
								}
							}
						}
					}
					else {
						if(psc < 5) {
							for(i=0; i<100; i++) {
								if( shots[i].type == 0) {
									shots[i].type = 1;
									shots[i].x = player->pos.x + player->pos.w / 2  - 2;
									shots[i].y = player->pos.y - 2;
									shots[i].vx = 0;
									shots[i].vy = -4;
									shots[i].alive = 1;
								
									Mix_PlayChannel(1,shot_snd, 0);
								
									break;
								}
							}
						}
					}
					fire = 1;
				}
				else {
					if(player->timer[0] < 300) player->timer[0] += 2;
					if(player->timer[0] > 300) player->timer[0] = 300;
				}
			}
		}
		else {
			fire = 0;
			if(player->timer[0] < 300 && player->timer[0] >= 0) player->timer[0] -= 4;
			if(player->timer[0] < 0) player->timer[0] = 0;
		}

		draw_background();   
		
		DrawObjects();

		MoveObjects();		

		if(CheckCollisions()) {
			player->alive = 0;
			Explode(2, player->pos);
		}
		else
			if(!player->alive && (pc == 0)) done = 1;

		if(atc[0] < 5) {
			if(SDL_GetTicks() > spawntimer) {
				for(i=0; i<100; i++) {
					if(game_obs[i].type == 0) {
						game_obs[i].type = 2;
						game_obs[i].pos.x = rand()%(640 - alien->w);
						game_obs[i].pos.y = -alien->h * 2;
						game_obs[i].pos.h = alien->h;
						game_obs[i].pos.w = alien->w;
						game_obs[i].state = 0;
						game_obs[i].mask = cd_alien;
						break;
					}
				}
				spawntimer = SDL_GetTicks() + 500;
			}
		}
		if(SDL_GetTicks() > starttime + 15000) {
			if(!(rand()%50) && atc[1] == 0) {
				for(i=0; i<100; i++) {
					if(game_obs[i].type == 0) {
						game_obs[i].type = 3;
						game_obs[i].pos.x = screen->w + alien2[0]->w;
						game_obs[i].pos.y = rand() % alien2[0]->h;
						game_obs[i].pos.h = alien2[0]->h;
						game_obs[i].pos.w = alien2[0]->w;
						game_obs[i].timer[0] = SDL_GetTicks() + rand()%250;
						game_obs[i].timer[1] = SDL_GetTicks() + rand()%250;
						game_obs[i].state = 0;
						game_obs[i].mask = cd_alien2[0];
						break;
					}
				}
			}
		}

		if(SDL_GetTicks() > starttime + 30000) {
			if(!(rand()%100) && atc[2] == 0) {
				k = (screen->h / 4) - rand()%(screen->h / 4);
				for(j=0; j<5; j++) {
					for(i=0; i<100; i++) {
						if(game_obs[i].type == 0) {
							game_obs[i].type = 5;
							game_obs[i].pos.x = j * alien3[1]->w ;
							game_obs[i].pos.y = k + rand() % alien3[1]->h;
							game_obs[i].pos.h = alien3[1]->h;
							game_obs[i].pos.w = alien3[1]->w;
							game_obs[i].timer[0] = SDL_GetTicks() + rand()%250 + i * 250;
							game_obs[i].state = 0;
							game_obs[i].mask = cd_alien3[1];
							break;
						}
					}
				}
			}
		}

		if(SDL_GetTicks() > starttime + 45000) {
			if(!(rand()%100) && atc[7] == 0) {
				for(i=0; i<100; i++) {
					if(game_obs[i].type == 0) {
						game_obs[i].type = 7;
						game_obs[i].pos.x = rand()%(screen->w - alien4[0]->w);
						game_obs[i].pos.y = -alien4[0]->h * 2;
						game_obs[i].pos.h = alien4[0]->h;
						game_obs[i].pos.w = alien4[0]->w;
						game_obs[i].mask = cd_alien4[0];
						break;
					}
				}
			}
		}
	
		if(SDL_GetTicks() > pupspawn) {
			for(i=0; i<100; i++) {
				if(game_obs[i].type == 0) {
					pupspawn = SDL_GetTicks() + 10000 + (rand()%5000);
					game_obs[i].type = 100 + rand() % 5;
					game_obs[i].pos.x = rand()%(screen->w - powerup[0]->w);
					game_obs[i].pos.y = -powerup[0]->h;
					game_obs[i].pos.h = powerup[0]->h;
					game_obs[i].pos.w = powerup[0]->w;
					game_obs[i].mask = cd_powerup[0];
					break;
				}
			}
		}


		sprintf(buffer, "score: %i", score );
		blitstr(screen, 0,0, buffer);

		//dst.x = 0;
		//dst.y = screen->h - 16;
		//dst.w = screen->w;
		//dst.h = 16;
		//SDL_FillRect(screen, &dst, SDL_MapRGB(screen->format, 0xff,0xff,0xff));

		dst.x = 8;
		dst.y = screen->h - meter->h + 8;
		dst.w = player->timer[0] * (624) / 300;
		dst.h = 16;

		src.x = 0;
		src.y = 0;
		src.w = dst.w;
		src.h = meter2->h;

		//SDL_FillRect(screen, &dst, SDL_MapRGB(screen->format, 0x88,0x0,0x0));
		SDL_BlitSurface(meter2, &src, screen, &dst);

		if(player->state & 1) {
			dst.x = 0;
			dst.y = screen->h - meter->h - powerup[1]->h;
			dst.w = powerup[1]->w;
			dst.h = powerup[1]->h;
			SDL_BlitSurface(powerup[1], NULL, screen, &dst);
		}
		if(player->state & 2) {
			dst.x = powerup[2]->w;
			dst.y = screen->h - meter->h - powerup[2]->h;
			dst.w = powerup[2]->w;
			dst.h = powerup[2]->h;
			SDL_BlitSurface(powerup[2], NULL, screen, &dst);
		}


		dst.x = 0;
		dst.y = screen->h - meter->h;
		dst.w = meter->w;
		dst.h = meter->h;
		SDL_BlitSurface(meter, NULL, screen, &dst);

		if(keys[SDLK_s] == SDL_PRESSED ) {
			SDL_SaveBMP(screen, "ShipDemo.bmp");
		}

		SDL_Flip(screen);

		Wait_Frame();

		if(wasteshots) {
			wasteshots = 0;
			SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0xff,0xff,0xff));
		}
		else 
			SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0,0,0));


		tiley--;

	} while(!done);

}

void TitleScreen(void) {
	char buffer[255];
	SDL_Event event;



	while(1) {
		SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0,0,0));
		if(score > hiscore) {
			hiscore = score;
			blitstr(screen, 0,Font->h * 3, "New High Score");
		}

		sprintf(buffer, "High Score for session is %i", hiscore);
		blitstr(screen, 0,Font->h*2, buffer);
		blitstr(screen, 0,0, "Certain Death Via Space Things");
		blitstr(screen, 0,screen->h / 2, "Press almost any key to begin");
		SDL_Flip(screen);

		while(SDL_PollEvent(&event)) {
			if ( event.type == SDL_QUIT )
				return;
			if ( event.type == SDL_KEYDOWN ) {
				if(event.key.keysym.sym == SDLK_ESCAPE)
					return;
				RunGame();
			}
		}
	}
}


int main(int argc, char *argv[]) {
	Uint32 flags=0;

	/* Some object types 

	0 No object
	1 player's ship
	2 alien (bug)
	3 alien (lobster left)
	4 alien (lobster right)

    100 powerup
 
	**********************/

	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0 ){
		fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}

	flags |= SDL_FULLSCREEN;
	flags |= SDL_DOUBLEBUF;

	if( (screen = SDL_SetVideoMode( 640, 480, 0, flags)) == NULL ) {
		fprintf(stderr, "Couldn't set video mode: %s\n", SDL_GetError());
		exit(1);
	}

	if ( Mix_OpenAudio(22050, AUDIO_S16, 1, 1024) < 0 ) 
	{
		fprintf(stderr,
			"Warning: Couldn't set 22050 Hz 16-bit audio\n- Reason: %s\n",
			SDL_GetError());
	}

	SDL_ShowCursor(0);

	srand(SDL_GetTicks());

	LoadData();
   
	TitleScreen();
	
	FreeData();

	Mix_CloseAudio();

	SDL_Quit();

	return(0);
}



